@@ -53,6 +53,7 @@
 
 #define MAX_NOTIFICATIONS 20
 
+#define WIDTH 400
 #define IMAGE_SIZE 48
 #define IDLE_SECONDS 30
 #define NOTIFICATION_BUS_NAME      "org.freedesktop.Notifications"
@@ -87,8 +88,8 @@ typedef struct {
 	guint id;
 	GtkWindow* nw;
 	Window src_window_xid;
-	guint has_timeout: 1;
-	guint paused: 1;
+	gboolean has_timeout;
+	gboolean paused;
 } NotifyTimeout;
 
 typedef struct {
@@ -129,12 +130,32 @@ static void monitor_notification_source_windows(NotifyDaemon* daemon, NotifyTime
 
 G_DEFINE_TYPE(NotifyDaemon, notify_daemon, G_TYPE_OBJECT);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+notify_daemon_get_preferred_width (GtkWidget *widget,
+                                   gint *min_width,
+                                   gint *nat_width)
+{
+        if (nat_width != NULL) {
+            *nat_width = WIDTH;
+        }
+}
+#endif
+
 static void notify_daemon_class_init(NotifyDaemonClass* daemon_class)
 {
 	GObjectClass* object_class = G_OBJECT_CLASS(daemon_class);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (daemon_class);
+#endif
+
 	object_class->finalize = notify_daemon_finalize;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+    widget_class->get_preferred_width = notify_daemon_get_preferred_width;
+#endif
+
 	g_type_class_add_private(daemon_class, sizeof(NotifyDaemonPrivate));
 }
 
@@ -273,11 +294,7 @@ static void create_screens(NotifyDaemon* daemon)
 
 		daemon->priv->screens[i] = g_new0(NotifyScreen, 1);
 
-		#if GTK_CHECK_VERSION(3, 0, 0)
 			daemon->priv->screens[i]->workarea_atom = XInternAtom(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "_NET_WORKAREA", True);
-		#else
-			daemon->priv->screens[i]->workarea_atom = XInternAtom(GDK_DISPLAY(), "_NET_WORKAREA", True);
-		#endif
 
 		gdkwindow = gdk_screen_get_root_window(screen);
 		gdk_window_add_filter(gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->priv->screens[i]);
@@ -312,7 +329,7 @@ static void on_popup_location_changed(GSettings *settings, gchar *key, NotifyDae
 	{
 		int j;
 
-		for (j = 0; j < daemon->priv->n_screens; j++)
+		for (j = 0; j < daemon->priv->screens[i]->n_stacks; j++)
 		{
 			NotifyStack* stack;
 			stack = daemon->priv->screens[i]->stacks[j];
@@ -332,12 +349,12 @@ static void notify_daemon_init(NotifyDaemon* daemon)
 
 	daemon->gsettings = g_settings_new (GSETTINGS_SCHEMA);
 
+	g_signal_connect (daemon->gsettings, "changed::" GSETTINGS_KEY_POPUP_LOCATION, G_CALLBACK (on_popup_location_changed), daemon);
+
 	location = g_settings_get_string (daemon->gsettings, GSETTINGS_KEY_POPUP_LOCATION);
 	daemon->priv->stack_location = get_stack_location_from_string(location);
 	g_free(location);
 
-	g_signal_connect (daemon->gsettings, "changed::" GSETTINGS_KEY_POPUP_LOCATION, G_CALLBACK (on_popup_location_changed), daemon);
-
 	daemon->priv->n_screens = 0;
 	daemon->priv->screens = NULL;
 
@@ -367,7 +384,7 @@ static void destroy_screens(NotifyDaemon* daemon)
 		gdkwindow = gdk_screen_get_root_window(screen);
 		gdk_window_remove_filter(gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->priv->screens[i]);
 
-		for (j = 0; i < daemon->priv->screens[i]->n_stacks; j++)
+		for (j = 0; j < daemon->priv->screens[i]->n_stacks; j++)
 		{
 			notify_stack_destroy(daemon->priv->screens[i]->stacks[j]);
 			daemon->priv->screens[i]->stacks[j] = NULL;
@@ -1036,8 +1053,9 @@ static void window_clicked_cb(GtkWindow* nw, GdkEventButton* button, NotifyDaemo
 static void url_clicked_cb(GtkWindow* nw, const char *url)
 {
 	NotifyDaemon* daemon;
-	char* escaped_url;
-	char* cmd = NULL;
+	gchar *escaped_url;
+	gchar *cmd = NULL;
+	gchar *found = NULL;
 
 	daemon = NW_GET_DAEMON(nw);
 
@@ -1046,15 +1064,15 @@ static void url_clicked_cb(GtkWindow* nw, const char *url)
 
 	escaped_url = g_shell_quote (url);
 
-	if (g_find_program_in_path ("gvfs-open") != NULL)
+	if ((found = g_find_program_in_path ("gvfs-open")) != NULL)
 	{
 		cmd = g_strdup_printf ("gvfs-open %s", escaped_url);
 	}
-	else if (g_find_program_in_path ("xdg-open") != NULL)
+	else if ((found = g_find_program_in_path ("xdg-open")) != NULL)
 	{
 		cmd = g_strdup_printf ("xdg-open %s", escaped_url);
 	}
-	else if (g_find_program_in_path ("firefox") != NULL)
+	else if ((found = g_find_program_in_path ("firefox")) != NULL)
 	{
 		cmd = g_strdup_printf ("firefox %s", escaped_url);
 	}
@@ -1063,6 +1081,7 @@ static void url_clicked_cb(GtkWindow* nw, const char *url)
 		g_warning ("Unable to find a browser.");
 	}
 
+	g_free (found);
 	g_free (escaped_url);
 
 	if (cmd != NULL)
@@ -1107,11 +1126,7 @@ static gboolean fullscreen_window_exists(GtkWidget* nw)
 	WnckWorkspace* wnck_workspace;
 	GList* l;
 
-	#if GTK_CHECK_VERSION (3, 0, 0) || ((GDK_PIXBUF_MAJOR == 2) && (GDK_PIXBUF_MINOR < 22))
 		wnck_screen = wnck_screen_get(GDK_SCREEN_XNUMBER(gdk_window_get_screen(gtk_widget_get_window(nw))));
-	#else
-		wnck_screen = wnck_screen_get(GDK_SCREEN_XNUMBER(gdk_drawable_get_screen(GDK_DRAWABLE(GTK_WIDGET(nw)->window))));
-	#endif
 
 	wnck_screen_force_update (wnck_screen);
 
