@@ -174,11 +174,7 @@ paint_window(GtkWidget *widget,
 #endif
 
 	if (windata->width == 0) {
-#if GTK_CHECK_VERSION(3, 0, 0)
 		gtk_widget_get_allocation(windata->win, &allocation);
-#else
-		allocation = windata->win->allocation;
-#endif
 		windata->width = allocation.width;
 		windata->height = allocation.height;    
 	}
@@ -224,25 +220,17 @@ paint_window(GtkWidget *widget,
 #endif
 	}
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 	context = gdk_cairo_create(gtk_widget_get_window(widget));
-#else 
-	context = gdk_cairo_create(widget->window);
-#endif
 
 	cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	surface = cairo_surface_create_similar(cairo_get_target(context),
 										   CAIRO_CONTENT_COLOR_ALPHA,
 										   windata->width,
 										   windata->height);
+#if GTK_CHECK_VERSION (3, 0, 0)
 	cairo_set_source_surface (cr, surface, 0, 0);
 #else
-	surface = cairo_surface_create_similar(cairo_get_target(context),
-										   CAIRO_CONTENT_COLOR_ALPHA,
-										   widget->allocation.width,
-										   widget->allocation.height);
 	cr = cairo_create(surface);
 #endif
 
@@ -290,11 +278,7 @@ countdown_expose_cb(GtkWidget *pie,
 	cairo_t *cr;
 #endif
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 	context = gdk_cairo_create(gtk_widget_get_window(pie));
-#else 
-	context = gdk_cairo_create(pie->window);
-#endif
 
 	cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
 #if GTK_CHECK_VERSION (3, 0, 0)
@@ -487,6 +471,9 @@ create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(vbox), windata->summary_label, FALSE, FALSE, 0);
 	gtk_misc_set_alignment(GTK_MISC(windata->summary_label), 0, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(windata->summary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
+#endif
 
 	atkobj = gtk_widget_get_accessible(windata->summary_label);
 	atk_object_set_description(atkobj, "Notification summary text.");
@@ -495,6 +482,9 @@ create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(vbox), windata->body_label, FALSE, FALSE, 0);
 	gtk_misc_set_alignment(GTK_MISC(windata->body_label), 0, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(windata->body_label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->body_label), PANGO_WRAP_WORD_CHAR);
+#endif
 	g_signal_connect(G_OBJECT(windata->body_label), "activate-link",
                          G_CALLBACK(activate_link), windata);
 
@@ -586,11 +576,7 @@ add_notification_action(GtkWindow *nw, const char *text, const char *key,
 
 	g_assert(windata != NULL);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-	if (gtk_widget_get_visible(windata->actions_box))
-#else 
-	if (!GTK_WIDGET_VISIBLE(windata->actions_box))
-#endif
+	if (!gtk_widget_get_visible(windata->actions_box))
 	{
 		GtkWidget *alignment;
 
@@ -627,11 +613,7 @@ add_notification_action(GtkWindow *nw, const char *text, const char *key,
 	buf = g_strdup_printf("stock_%s", key);
 	pixbuf = gtk_icon_theme_load_icon(
 		gtk_icon_theme_get_for_screen(
-#if GTK_CHECK_VERSION(3, 0, 0)
 			gdk_window_get_screen(gtk_widget_get_window(GTK_WIDGET(nw)))),
-#else 
-			gdk_drawable_get_screen(GTK_WIDGET(nw)->window)),
-#endif
 		buf, 16, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
 	g_free(buf);
 
@@ -668,11 +650,7 @@ clear_notification_actions(GtkWindow *nw)
 
 	gtk_widget_hide(windata->actions_box);
 	gtk_container_foreach(GTK_CONTAINER(windata->actions_box),
-#if GTK_CHECK_VERSION (3, 0, 0)
-						  (GtkCallback)g_object_unref, NULL);
-#else
-						  (GtkCallback)gtk_object_destroy, NULL);
-#endif
+						  (GtkCallback)gtk_widget_destroy, NULL);
 }
 
 /* Move notification window */
